
package com.sleepless.misc;


import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.net.*;
import java.io.*;
import java.util.*;


public class AppletFrame implements Runnable, AppletStub, AppletContext, WindowListener
{


	public static void main(String [] args)
	{
		try
		{
			Applet applet = (Applet)Class.forName(args[0]).newInstance();
			new AppletFrame( applet, args,
				"AppletFrame",
				"Sleepless Software Inc.",
				"http://www.sleepless.com"
				);
		}
		catch(IllegalAccessException e) { e.printStackTrace(); }
		catch(ClassNotFoundException e) { e.printStackTrace(); }
		catch(InstantiationException e) { e.printStackTrace(); }
	}


	private String[] args = null;
	private String name;
	private Applet applet;
	private Label label = null;
	private static final String PPREFIX = "param.";
	private Frame frame;
	private String browser = "AppletFrame";
	private String version = "V1.0";


	public AppletFrame(Applet applet, String[] args, String title, String vendor, String vendorURL) {

		this.applet = applet;
		this.args = args;
		applet.setStub( this );

		name = applet.getClass().getName();

		frame = new Frame(title);

		Properties props = System.getProperties();
		props.put( "browser", browser);
		props.put( "browser.version", version);
		props.put( "browser.vendor", vendor);
		props.put( "browser.vendor.url", vendorURL);

		if ( args != null )
			parseArgs( args, props );

		frame.setLayout( new BorderLayout() );
		frame.add( "Center", applet );

		frame.pack();
		frame.validate();

		applet.setSize( (int)atol(getParameter("WIDTH")), (int)atol(getParameter("HEIGHT")) );

		frame. addWindowListener(this);

		new Thread( this ).start();
	}

	private static long atol(String s)
	{
		try { return Long.parseLong(s); } catch(NumberFormatException e) { }
		return 0;
	}
    
	private static void parseArgs( String[] args, Properties props )
	{
		for ( int i = 0; i < args.length; ++i )
		{
			String arg = args[i];
			int ind = arg.indexOf( '=' );
			if ( ind == -1 )
				props.put( PPREFIX + arg.toLowerCase(), "" );
			else
				props.put( PPREFIX + arg.substring( 0, ind ).toLowerCase(), arg.substring( ind + 1 ) );
		}
	}

	public void run()
	{
		frame.setVisible(true);
		showStatus( name + " Initializing applet ..." );
		applet.init();
		frame.validate();
		showStatus( name + " Starting applet ..." );
		applet.start();
		frame.validate();
		showStatus( name + " Applet running ..." );
	}

	public boolean isActive() {
		return true;
	}

	public URL getDocumentBase() {
		String dir = System.getProperty( "user.dir" );
		String urlDir = dir.replace( File.separatorChar, '/' );
		try {
			return new URL( "file:" + urlDir + "/");
		}
		catch ( MalformedURLException e ) {
			return null;
		}
	}
    
	public URL getCodeBase() {
		String path = System.getProperty( "java.class.path" );
		Enumeration st = new StringTokenizer( path, File.pathSeparator);
		while ( st.hasMoreElements() )
		{
			String dir = (String) st.nextElement();
			String filename = dir + File.separatorChar + name + ".class";
			File file = new File( filename );
			if ( file.exists() )
			{
				try
				{
					String canPath = new File(dir).getCanonicalPath();
					String urlDir = canPath.replace( File.separatorChar, '/' );
					URL u = new URL( "file:" + urlDir + "/" );
					return u;
				}
				catch ( MalformedURLException e )
				{
					return null;
				}
				catch (IOException e)
				{
					return null;
				}
			}
		}
		return null;
	}
    
	public String getParameter( String name ) {
		return System.getProperty( PPREFIX + name.toLowerCase() );
	}
    
	public void appletResize( int width, int height ) {
		Insets i = frame.getInsets();
		int w = width + i.left + i.right;
		int h = height + i.top + i.bottom;
		frame.setSize(w, h);
		applet.setLocation(i.left, i.top);
	}

	public AppletContext getAppletContext() {
		return this;
	}
    
	public AudioClip getAudioClip( URL url ) {
		try {
			return new sun.applet.AppletAudioClip( url );
		}
		catch (java.lang.NoClassDefFoundError e) { }
		return null;
	}

	public Image getImage( URL url ) {
		Toolkit tk = Toolkit.getDefaultToolkit();
		try
		{
			ImageProducer prod = (ImageProducer) url.getContent();
			return tk.createImage( prod );
		}
		catch(IOException e) { }
		return null;
	}
    
	public Applet getApplet( String name ) {
		if ( name.equals( this.name ) )
			return applet;
		return null;
	}
    
	public Enumeration getApplets() {
		Vector v = new Vector();
		v.addElement( applet );
		return v.elements();
	}
    
	public void showDocument( URL url ) {
	}
    
	public void showDocument( URL url, String target ) {
	}
	    
	public void showStatus( String status ) {
		if ( label != null )
			label.setText( status );
	}

	public void windowOpened(WindowEvent e) {
	}

	public void windowClosing(WindowEvent e) {
		System.exit(0);
	}

	public void windowClosed(WindowEvent e) {
	}

	public void windowIconified(WindowEvent e) {
	}

	public void windowDeiconified(WindowEvent e) {
	}

	public void windowActivated(WindowEvent e) {
	}

	public void windowDeactivated(WindowEvent e) {
	}

}


