
/* Copyright 1998-2006 Sleepless Software Inc.  All Rights Reserved */

/*
   ************************************************************************* 
   WARNING: THIS CODE HAS A C++ COUNTERPART. DON'T CHANGE THIS WITHOUT ALSO
   MAKING THE CORRESPONDING CHANGES TO THE C++ VERSION.
   ************************************************************************* 

	This file contains two functions:

		void cloak(unsigned char *buf, int len)
		void decloak(unsigned char *buf, int len)

	They are designed to take any buffer of bytes and
	scramble up the bits so that the contents are concealed
	from casual inspection.

	This is not intended to be "strong" security by any means.
*/

class Cloak
{

   // generator polynomial
   private static final int poly = 0x1021; // x16 + x12 + x5 + 1 generator polynomial
   // 0x8408 used in European X.25

   // lookup table for fast computation.
   private static int[] cloak_crc_table = new int[256];
   static {
      // initialise table
      for ( int i=0; i<256; i++ )
         {
         int fcs = 0;
         int d = i << 8;
         for ( int k=0; k<8; k++ )
            {
            if ( ((fcs ^ d) & 0x8000) != 0 )
               {
               fcs = ( fcs << 1 ) ^ poly;
               }
            else
               {
               fcs = ( fcs << 1 );
               }
            d <<= 1;
            fcs &= 0xffff;
            }
         cloak_crc_table[i] = fcs;
         }
      }


	static void cloak(byte [] buf, int len)
	{
		int crc = 0x1badbabe;

		for(int i = 0; i < len; i++)
		{
			int ch = buf[i];
			buf[i] = (byte)(ch ^ crc);
			crc = cloak_crc_table[(crc ^ ch) & 0xff] ^ (crc >> 8);
		}
	}

	static void decloak(byte [] buf, int len)
	{
		int crc = 0x1badbabe;

		for(int i = 0; i < len; i++)
		{
			int ch = buf[i] ^ crc;
			buf[i] = (byte)ch;
			crc = cloak_crc_table[(crc ^ ch) & 0xff] ^ (crc >> 8);
		}
	}


/*
	static void p(String s)
	{
		System.out.print(s);
	}


	static final char[] hex = "0123456789ABCDEF".toCharArray();
	static void hexdump(byte[] buf,int len) {
		byte b;
		for(int i=0; i < len; i++) {
			b = buf[i];
			System.out.print(" "+ hex[(b>>>4)&0xf] + hex[(b&0xf)&0xf] );
		}
	}

	public static void main(String [] args)
	{
		int l;

		if (args.length < 1)
			System.exit(1);

		String ss = args[0];
		byte [] s = ss.getBytes();
		l = ss.length();
		p("original :"); hexdump(s, l); p("\n");
		cloak(s, l);
		p("cloaked  :"); hexdump(s, l); p("\n");
		decloak(s, l);
		p("decloaked:"); hexdump(s, l); p("\n");
	}
*/
		
}





