#ifndef com_sleepless_misc_crc32
#define com_sleepless_misc_crc32


/* The CRC class here is built from the guts of some great
 * public domain CRC code written by Michael Barr.
 * Example usage:
 *
 *		static Crc32 crc32;
 *		unsigned long int sum = crc32.crcFast(buf, size);
 */

class Crc32
{
	unsigned long POLYNOMIAL;
	unsigned long INITIAL_REMAINDER;
	unsigned long FINAL_XOR_VALUE;
	int REFLECT_DATA;
	int REFLECT_REMAINDER;
	unsigned long  crcTable[256];
	int WIDTH;
	unsigned long TOPBIT;
	unsigned long  f_remainder;

	unsigned long reflect(unsigned long data, unsigned char nBits)
	{
		unsigned long  reflection = 0x00000000;
		unsigned char  bit;

		for (bit = 0; bit < nBits; ++bit)
		{
			if (data & 0x01)
			{
				reflection |= (1 << ((nBits - 1) - bit));
			}
			data = (data >> 1);
		}
		return (reflection);
	}

	unsigned char reflectData(unsigned char b)
	{
		if(REFLECT_DATA)
			return (unsigned char)reflect(b, 8);
		return b;
	}

	unsigned long reflectRemainder(unsigned long n)
	{
		if(REFLECT_REMAINDER)
			return (unsigned long)reflect(n, WIDTH);
		return n;
	}

	public:

	Crc32(void)
	{
		POLYNOMIAL = 0x04C11DB7;
		INITIAL_REMAINDER = 0xffffffff;
		FINAL_XOR_VALUE = 0xffffffff;
		REFLECT_DATA = 1;
		REFLECT_REMAINDER = 1;
		WIDTH = (8 * sizeof(unsigned long));
		TOPBIT = (1 << (WIDTH - 1));

		unsigned long  remainder;
		int			   dividend;
		unsigned char  bit;
		for(dividend = 0; dividend < 256; ++dividend)
		{
			remainder = dividend << (WIDTH - 8);
			for (bit = 8; bit > 0; --bit)
			{
				if (remainder & TOPBIT)
				{
					remainder = (remainder << 1) ^ POLYNOMIAL;
				}
				else
				{
					remainder = (remainder << 1);
				}
			}
			crcTable[dividend] = remainder;
		}
	}


	void start()
	{
		f_remainder = INITIAL_REMAINDER;
	}
	void feed(unsigned char c)
	{
		unsigned char  data;
		data = reflectData(c) ^ (unsigned char)(f_remainder >> (WIDTH - 8));
		f_remainder = crcTable[data] ^ (f_remainder << 8);
	}
	unsigned long finish()
	{
		return (reflectRemainder(f_remainder) ^ FINAL_XOR_VALUE);
	}


	unsigned long crcFast(unsigned char const message[], int nBytes)
	{
		int            byte;
		start();
		for (byte = 0; byte < nBytes; ++byte)
		{
			feed(message[byte]);
		}
		return finish();
	}

};


#endif	// com_sleepless_misc_crc32
