
#ifndef com_sleepless_deflater_h
#define com_sleepless_deflater_h


#include "../jumbo/zlib-1.1.3/zlib.h"

#include <stdio.h>

struct Deflater
{

	int (*writer)(void *, char *, int);
	void *whandle;

    z_stream stream;
    char z_out[10000];

	bool initialized;
	bool stopped;

	char *error;



	char *getError()
	{
		return error;
	}


	Deflater(int (*wf)(void *, char *, int), void *wh)
	{
		stream.zalloc = (alloc_func)0;
		stream.zfree = (free_func)0;
		stream.opaque = (voidpf)0;

		stream.next_in = 0;
		stream.avail_in = 0;

		stream.next_out = (Bytef *)z_out;
		stream.avail_out = sizeof(z_out);

		writer = wf;
		whandle = wh;

		initialized = false;
		stopped = false;

		error = 0;
	}


	~Deflater()
	{
		if(!stopped)
		{
			// Flush out the remainder
			int w;
			int r;
			while(true)
			{
				if(stream.avail_out == 0)
				{
					w = writer(whandle, z_out, sizeof(z_out));
					if(w == -1)
					{
						stopped = true;
						error = "write error";
						break;
					}
					stream.next_out = (Bytef *)z_out;
					stream.avail_out = sizeof(z_out);
				}

				r = deflate(&(stream), Z_FINISH);

				if(r == Z_STREAM_END)
				{
					stopped = true;
					break;
				}
			}

			// Scrape out whatever is left in the hopper.
			int n = sizeof(z_out) - stream.avail_out;
			w = writer(whandle, z_out, n);
			if(w != n)
			{
				stopped = true;
				error = "write error";
			}
		}

		deflateEnd(&stream);

	}


	int write(const char *buf, unsigned len)
	{
		if(stopped)
			return 0;

		if(!initialized)
		{
			if(deflateInit(&stream, 1) != Z_OK)
			{
				error = "deflater won't start";
				return 0;
			}
			initialized = true;
		}

		int oti = stream.total_in;

		stream.next_in = (Bytef *)buf;
		stream.avail_in = len;

		int r;
		int w;

		while(stream.avail_in > 0)
		{
			if(stream.avail_out == 0)
			{
				w = writer(whandle, z_out, sizeof(z_out));
				if(w == -1)
				{
					stopped = true;
					error = "write error";
					break;
				}
				stream.next_out = (Bytef *)z_out;
				stream.avail_out = sizeof(z_out);
			}

			r = deflate(&(stream), Z_NO_FLUSH);

			if(r != Z_OK)
			{
				stopped = true;
				error = "deflater jammed";
				break;
			}
		}

		return stream.total_in - oti;
	}

};




#ifdef DEFLATE_TEST

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(MSDOS) || defined(OS2) || defined(WIN32)
#  include <fcntl.h>
#  include <io.h>
#  define SET_BINARY_MODE(file) setmode(fileno(file), O_BINARY)
#else
#  define SET_BINARY_MODE(file)
#endif


static int fpWriter(void *handle, char *buf, int len)
{
	FILE *fp = (FILE *)handle;
	int n = fwrite(buf, 1, len, fp);
	if(ferror(fp))
		return -1;
//fprintf(stderr, "wrote %d bytes:  %x %x\n", n, buf[0]&0xff, buf[1]&0xff );
	return n;
}


static char buf[10000];

int main(int argc, char **argv)
{
	SET_BINARY_MODE(stdin);
	SET_BINARY_MODE(stdout);

	Deflater deflater(fpWriter, stdout);
	while(true)
	{
		int r = fread(buf, 1, sizeof(buf), stdin);
		if(ferror(stdin))
		{
			fprintf(stderr, "read error\n");
			exit(1);
		}
		int w = deflater.write(buf, r);
		if(deflater.getError())
		{
			fprintf(stderr, "%s\n", deflater.getError());
			exit(1);
		}
		if(w != sizeof(buf))
			break;
	}

	return 0;
}

#endif


#endif


