#ifndef com_sleepless_imagecache_cpp
#define com_sleepless_imagecache_cpp

// Copyright 2004 Sleepless Software Inc.  All Rights Reserved

/*
	ImageCache()
	ImageCache(const char *p)
	void setPath(const char *p)
	void clear()
	Image *tryLoad(const char *f, const char *e)
	Image *load(const char *f)
	Image *get(const char *name)
*/



struct ImageCache
{
	TagArray imgs;
	char path[300];


	ImageCache()
	{
		setPath("");
	}

	ImageCache(const char *p)
	{
		setPath(p);
	}


	/* Set the path for where to look for images */
	void setPath(const char *p)
	{
		STR_CPY(path, p);
	}

	/* Delete all images from the cache.
		Any pointers returned from the load routines will become invalid. */
	void clear()
	{
		while(imgs.length() > 0)
			delete (Image *)imgs.get(0);
	}


	/* Attempt to load the file with the given name and the given extension.
		Return the image or null if unsuccessful.
		Does NOT cache images ... this will always load and create a
		new instance of the image structure.  */
	Image *tryLoad(const char *f, const char *e)
	{
		Image *img = 0;
		char *p = new char[300];
		if(p)
		{
			Str::copy(p, path, 300);
			Str::append(p, "/", 300);
			Str::append(p, f, 300);
			Str::append(p, ".", 300);
			Str::append(p, e, 300);
			img = new Image(p);
			if(img)
			{
				if(img->width == 0)
				{
					delete img;
					img = 0;
				}
			}
			delete p;
		}
		return img;
	}


	/* Attempts to load an image from disk and returns a pointer to it
		or null if it fails.
		Attempts several different extensions using tryLoad().
		Does NOT cache images ... this will always load and create a
		new instance of the image structure.  */
	Image *load(const char *f)
	{
		Image *img = 0;
		if((f != 0) || (f[0] != 0))
		{
			/* we have what looks like a valid file name */
			img = tryLoad(f, "ppm");
			if(!img)
			{
				img = tryLoad(f, "bmp");
				if(!img)
				{
					img = tryLoad(f, "tga");
				}
			}
		}
		DASSERT(img);
		DASSERT(img->width);
		return img;
	}


	/* Returns a pointer to the named image from the cache, or null if
		not found.  This method caches images, so calling it twice with
		the same name, will return a pointer to the same image instance. */
	Image *get(const char *name)
	{
		Image *img = (Image *)imgs.get(name);
		if(!img)
		{
			img = load(name);
			if(img)
				imgs.put(name, img);
		}
		return img;
	}

};


#endif	// com_sleepless_imagecache_cpp
