#ifndef com_sleepless_inflater_cpp
#define com_sleepless_inflater_cpp

// Copyright 2002
// Sleepless Software Inc.
// All Rights Reserved



extern "C" {
#include "../../3rdparty/zlib-1.1.3/was-inflate.c"
#include "../../3rdparty/zlib-1.1.3/was-infcodes.c"
#include "../../3rdparty/zlib-1.1.3/was-adler32.c"
#include "../../3rdparty/zlib-1.1.3/was-zutil.c"
#include "../../3rdparty/zlib-1.1.3/was-inftrees.c"
#include "../../3rdparty/zlib-1.1.3/was-infutil.c"
#include "../../3rdparty/zlib-1.1.3/was-inffast.c"
#include "../../3rdparty/zlib-1.1.3/was-infblock.c"
/*
#include "zlib-1.1.3/was-inflate.c"
#include "zlib-1.1.3/was-infcodes.c"
#include "zlib-1.1.3/was-adler32.c"
#include "zlib-1.1.3/was-zutil.c"
#include "zlib-1.1.3/was-inftrees.c"
#include "zlib-1.1.3/was-infutil.c"
#include "zlib-1.1.3/was-inffast.c"
#include "zlib-1.1.3/was-infblock.c"
*/
}



struct Inflater
{

	int (*reader)(void *, char *, int);
	void *rhandle;

    z_stream stream;
    char z_in[10000];

	bool initialized;
	bool stopped;

	char *error;



	char *getError()
	{
		return error;
	}



	Inflater(int (*rf)(void *, char *, int), void *rh)
	{
		stream.zalloc = (alloc_func)0;
		stream.zfree = (free_func)0;
		stream.opaque = (voidpf)0;

		stream.next_in = 0;
		stream.avail_in = 0;

		stream.next_out = 0;
		stream.avail_out = 0;

		reader = rf;
		rhandle = rh;

		initialized = false;
		stopped = false;

		error = 0;
	}


	~Inflater()
	{
		inflateEnd(&stream);
	}


	int read(char *buf, unsigned len)
	{
		if(stopped)
			return 0;

		if(!initialized)
		{
			if(inflateInit(&stream) != Z_OK)
			{
				stopped = true;
				error = "inflater won't start";
				return 0;
			}
			initialized = true;
		}

		int oto = stream.total_out;

		stream.next_out = (Bytef *)buf;
		stream.avail_out = len;

		int r;
		while(stream.avail_out > 0)
		{
			if(stream.avail_in == 0)
			{
				r = reader(rhandle, z_in, sizeof(z_in));
				if(r == -1)
				{
					stopped = true;
					error = "read error";
					break;
				}
				if(r > 0)
				{
					stream.next_in = (Bytef *)z_in;
					stream.avail_in = r;
				}
			}

			r = inflate(&(stream), Z_SYNC_FLUSH);

			if(r == Z_STREAM_END)
			{
				stopped = true;
				break;
			}

			if(r != Z_OK)
			{
				stopped = true;
				error = "inflater jammed";
				break;
			}
		}

		return stream.total_out - oto;
	}

};





#ifdef INFLATE_TEST


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
#  include <fcntl.h>
#  include <io.h>
#  define SET_BINARY_MODE(file) setmode(fileno(file), O_BINARY)
#else
#  define SET_BINARY_MODE(file)
#endif


// This must return -1 on error.
// Otherwise it returns the amount read which should be 0 thru len inclusive.
// A short read or 0 indicates EOF.
static int fpReader(void *handle, char *buf, int len)
{
	FILE *fp = (FILE *)handle;
	int r = fread(buf, 1, len, fp);
	if(ferror(fp))
		return -1;
	return r;
}


static char buf[10000];

int main(int argc, char **argv)
{
	SET_BINARY_MODE(stdin);
	SET_BINARY_MODE(stdout);

	Inflater inflater(fpReader, stdin);
	while(true)
	{
		int r = inflater.read(buf, sizeof(buf));
		if(inflater.getError())
		{
			fprintf(stderr, "%s\n", inflater.getError());
			exit(1);
		}
		int w = fwrite(buf, 1, r, stdout);
		if(w != sizeof(buf))
			break;
	}

	return 0;
}

#endif


#endif	// com_sleepless_inflater_cpp


