#ifndef com_sleepless_install_cpp
#define com_sleepless_install_cpp

// Copyright 1998-2004 Sleepless Software Inc.  All Rights Reserved



#ifdef WIN32

/*
 This code is a little unusual.
 It is basically a complete win32 app with WinMain() and all, that
 implements a snazzy little installer app which installs from a
 jumbo file that is embedded in the .exe as a resource.

 To use, create a source file for your installer, such as "install.cpp".

 Add these lines:

	#define INSTALL_DIR "SSOL"
	#define APP_NAME "Sleepless Solitaire"

	static const char *shortCuts[] = {
	//	menu label		target				icon
		"Hint Book",	"hintbook.hlp",		"hb.ico",
		0,				0,					0,
	};

 Using the example above:

	 Files will be installed into "c:\Program Files\SSOL"

 Shortcuts will be created as such:
 	
	 Start -> Programs -> "Sleepless Solitaire" -> "Hint Book"
	 which links to "C:\Program Files\SSOL\hintbook.hlp"
	 and uses the icon from "C:\Program Files\SSOL\hb.ico"

 After defining those things, include this source file:

 		#include "../lib/install.cpp"

 Add your src file to an empty MSVC project.

 Create a few dialog resources with these text IDs:

 	Dialogs:

		"IDD_INSTALL"
			This dialog will be shown immediately before the installation
			starts and should have "Install" and "Cancel" buttons.
			The Install button should be ID_OK.
			The cancel button should be ID_CANCEL.
			It might say something like this:
				Welcome to the installation utility.  Please read
				the license carefully before proceeding.  If you do
				not agree to the terms of the license, click Cancel.
			You also need to have a "edit box" field in this dialog with
			ID 503.  The license text file will get loaded into this.
			Make sure it's read-only.
		"IDD_PROGRESS"
			This dialog must contain a progress bar item with ID 500.
			It is shown while the files are being extracted from the jumbo
			data.
		"IDD_SUCCESS"
			This dialog will be shown if the installation was successful.
			It should have only an "OK" button with ID_OK.
			It might say something like this:
				Congratulations!  Your software has been installed
				successfully.  You can start the program by looking
				under your start menu.
		"IDD_FAIL"
			This dialog will be shown if the installation failed.
			It should have only an "OK" button with ID_OK.
			It might say something like this:
				The installation was not completed normally.
				You will have to run the installation program at a 
				later time if you want to use this software.

 Create a a custom resource of type "JUM" with the
 ID 501 which contains the jumbo file data as created by jumbo.exe.
 This results in the jumbo file being compiled into the exe as a resource.

 Next, add a custom resource of type "LIC" with the ID 502
 which points to a text file containing your license agreement.  This
 will get loaded into the text field with ID 503 in the install dialog.

 Now, include the the zlib.lib in your link settings
 because the jumbo code will be looking for routines from that library.
 The zlib library is under the sleepless/jumbo dir.

 Lastly, you may need to change your code generation settings to 
 multi-threaded.

 That should do it.

*/ 




//////////////////////////////////////////////////////////////////////////

#include <windows.h>

#include "resource.h"

#include "windows.cpp"
#include "thread.cpp"
#include "jumboextractor.cpp"



static HINSTANCE instance = 0;
static HWND wProg = 0;
static HWND wBar = 0;

static const char *jumdata = 0;
static int jumsize = 0;
static int jumptr = 0;
static const char *installDir = INSTALL_DIR;
static const char *appName = APP_NAME;
//static const char *appExe = APP_EXE;
static Thread *jt = 0;



static int jReader(void *, char *buf, int len)
{
	int r = jumsize - jumptr;			// amount left to read
	if(r > len)							// if that's more than requested,
		r = len;						// only copy amount requested

	memcpy(buf, jumdata + jumptr, r);	// copy the bytes
	jumptr += r;

	SendMessage(wBar, PBM_SETPOS, (jumptr * 100) / jumsize, 0);

	// Sleep(50);

	return r;
}


struct Runner : Runnable
{
	void run()
	{
		// Decompress and extract my files
		jumptr = 0;
		JumboExtractor je(jReader, 0);
		je.extract(false);
		//if(je.extract(false) == -1)		// shouldn't ever fail

		SendMessage(wProg, WM_APP, 0, 0);	// Tell parent I'm done.
	}
} runner;


static BOOL CALLBACK wProgProc(HWND wnd, UINT msg, WPARAM w, LPARAM)
{
	switch(msg)
	{
	case WM_INITDIALOG:
		{
			wProg = wnd;
			wBar = GetDlgItem(wnd, 500);
			jt = new Thread(&runner);
			jt->start();
		}
		return TRUE;
	case WM_COMMAND:
		switch(LOWORD(w))
		{
		case IDCANCEL:
			EndDialog(wnd, -1);
			break;
		}
		return TRUE;
	case WM_APP:
		wBar = 0;
		EndDialog(wnd, 0);
	}
	return FALSE;
}


static BOOL CALLBACK wDlgProc(HWND wnd, UINT msg, WPARAM w, LPARAM)
{
	if(msg == WM_INITDIALOG)
	{
		HMODULE module = GetModuleHandle(NULL);
		if(module)
		{
			HRSRC res = FindResource(module, (const char *)502, "LIC");
			if(res)
			{
				int len =  SizeofResource(module,res);
				if(len > 1) 
				{
					HGLOBAL g = LoadResource(NULL, res);
					if(g)
					{
						const char *txt = (const char *)LockResource(g);
						if(txt)
						{
							SetDlgItemText(wnd, 503, txt);
						}
					}
				}
			}
		}
	}
	else
	if(msg == WM_COMMAND)
	{
		switch(LOWORD(w))
		{
		case IDOK:
			EndDialog(wnd, 0);
			break;
		case IDCANCEL:
			EndDialog(wnd, -1);
			break;
		}
		return 1;
	}
	return 0;
}


static void install()
{

	// Get a handle to the jumbo data, which is compiled in as
	// a custom resource type "JUM" in the .exe
	HMODULE module = GetModuleHandle(NULL);
	if(module == 0)
		exit(1);
	HRSRC res = FindResource(module, (const char *)501, "JUM");
	if(res == 0)
		exit(1);
	jumsize =  SizeofResource(module,res);
	if(jumsize == 0)
		exit(1);
	HGLOBAL g = LoadResource(NULL, res);
	if(g == 0)
		exit(1);
	jumdata = (const char *)LockResource(g);
	if(jumdata == 0)
		exit(1);


	// Change dir to "C:/programs files", or whatever is in registry
	char programs[MAX_PATH];
	Windows::getProgramFilesPath(programs, sizeof(programs));
	if(chdir(programs))
		exit(1);

	if(_mkdir(installDir))
	{
		// // XXX Perhaps I should tell the user about this, maybe 
		// // suggest that they delete a previously existing version?
		// if(false)
		// 	exit(1);
	}
	if(chdir(installDir))
		exit(1);


	// Put up progress dialog and do the extracting.
	int dr = DialogBox(instance, "IDD_PROGRESS", 0, (DLGPROC)wProgProc);
	if(dr != 0)
	{
		DialogBox(instance, "IDD_FAIL", 0, (DLGPROC)wDlgProc);
		exit(1);
	}

	const char **ptr = shortCuts;
	while(true)
	{
		const char *label = *ptr++;
		if(label == 0)
			break;
		const char *target = *ptr++;
		if(target == 0)
			break;
		const char *icon = *ptr++;
		if(icon == 0)
			break;
		Windows::addProgramsLink(appName, label, getcwd(0,1), target, icon);
	}

	// Hoo, ray.
	DialogBox(instance, "IDD_SUCCESS", 0, (DLGPROC)wDlgProc);

}


int PASCAL WinMain(HINSTANCE hInstance, 
				   HINSTANCE hPrevInstance, 
				   LPSTR lpCmdLine,
				   int nCmdShow)
{

	instance = hInstance;

	int d = DialogBox(instance, "IDD_INSTALL", 0, (DLGPROC)wDlgProc);
	if(d != 0)
		return 0;

	install();

	return 0;
}


#endif


#if defined(linux) || (defined(__APPLE__) && defined(__GNUC__))
#error "We don't have an installer like this for linux yet!"
#endif




#endif		// com_sleepless_install_cpp
