#ifndef com_sleepless_net_ircclient_h
#define com_sleepless_net_ircclient_h

// Copyright 2004 Sleepless Software Inc.  All Rights Reserved


#include "linesocket.cpp"
#include "../text/TokenShelf.cpp"



struct IRCClient
{
	char host[100];
	int port;
	char nick[32];
	LineSocket *lsock;
	char server[100];
	char version[100];
	char curChan[100];


	IRCClient()
	{
		host[0] = 0;
		port = 0;
		nick[0] = 0;
		lsock = 0;
		server[0] = 0;
		version[0] = 0;
		curChan[0] = 0;
	}


	~IRCClient()
	{
		disconnect();
	}


	int connect(const char *_host, int _port, const char *_nick)
	{
		disconnect();

		Str::copy(host, _host, sizeof(host));
		port = _port;
		Str::copy(nick, _nick, sizeof(nick));

		lsock = new LineSocket(512);
		int c = lsock->connect(host, port);
		if(c)
		{
			lsock->writeLine("NICK %s", nick);
			//lsock->writeLine("USER %s-user 0 * :%s-real", nick, nick);
			lsock->writeLine("USER %s foo bar :%s-real", nick, nick);
		}
		return c;
	}


	int isConnected()
	{
		return lsock ? lsock->isConnected() : 0;
	}


	void disconnect()
	{
		if(lsock)
		{
			delete lsock;
			lsock = 0;
		}
	}



	int read(
			void (*info)(const char *msg),
			void (*chat)(const char *from, const char *msg),
			void (*here)(const char *who),
			void (*join)(const char *who),
			void (*part)(const char *who)
			)
	{
		if(!isConnected())
			return 0;

		const char *line = lsock->readLine();

		if(!line)
			return 0;

		char from[32];
		char chan[100];
		char msg[512];

printf(">>>>>>>>>>>>>>>>> %50s\n", line);

		if(sscanf(line, ":%*s 004 %*s %s %s %*s %*s", server, version) == 2)
		{
			// :from 004 nick server version aoOirw abeiIklmn
			char buf[512];
			sprintf(buf, "IRC Server %s - version %s", server, version);
			if(info)
				info(buf);
		}
		else
		if(
			(sscanf(line, ":%31[^!]!%*s PRIVMSG %99s :%[^`]", from, chan, msg) == 3)
			||
			(sscanf(line, ":%31s PRIVMSG %99s :%[^`]", from, chan, msg) == 3)
			 )
		{
			// :from PRIVMSG chan :chat
			if(Str::equals(chan, curChan))
			{
				if(chat)
					chat(from, msg);
			}
		}
		else
		if(
			(sscanf(line, ":%31[^!]!%*s JOIN :%99[^ ]", from, chan) == 2)
			||
			(sscanf(line, ":%31s JOIN :%99[^ ]", from, chan) == 2)
			)
		{
			// :from JOIN :chan
			if(join)
				join(from);
			if(Str::equals(from, nick))
			{
				Str::copy(curChan, chan, sizeof(curChan));
//				char buf[100];
//				sprintf(buf, "Welcome to %s", chan);
//				info(buf);
				lsock->writeLine("WHO");
			}
		}
		else
		if(sscanf(line, ":%*[^ ] PART %99[^ ] :%31[^ ]", chan, msg) == 2)
		{
			// :from PART chan :who
			if(Str::equals(chan, curChan))
			{
				if(part)
					part(msg);
			}
		}
		else
		if(sscanf(line, ":%*s 353 %*s = %99s :%511[^`]", chan, msg) == 2)
		{
			// :from 353 to = chan :nick [nick ...]
			if(Str::equals(chan, curChan))
			{
				TokenShelf ts(msg, ' ');
				for(int i = 0; i < ts.length(); i++)
				{
					const char *n = ts.get(i);
					if(n[0] == '@')
						n++;
					if(here)
						here(n);
				}
			}
		}
		else
		if(sscanf(line, ":%*s 352 %*s %99s %*s %*s %*s %511s", chan, msg) == 2)
		{
			// response from 'who'
			// :from 352 to chan user host server nick H :0 realname
			if(Str::equals(chan, curChan))
				here(msg);
		}
		else
		if(sscanf(line, "PING :%s", msg) == 1)
		{
			// PING :server
			lsock->writeLine("PONG %s", msg);
		}

		return 1;
	}



	void write(const char *s)
	{
		if(!isConnected())
			return;
		lsock->writeLine(s);
	}

	void join(const char *chan)
	{
		if(!isConnected())
			return;
		if(curChan[0])
			lsock->writeLine("PART %s", curChan);
		lsock->writeLine("JOIN %s", chan);
	}


	void chat(const char *msg)
	{
		if(!isConnected())
			return;
		lsock->writeLine("PRIVMSG %s :%s", curChan, msg);
	}

};



#ifdef TEST_IRCCLIENT


#include <stdio.h>

#include "../sys/System.cpp"
#include "../sys/Thread.cpp"


static int done = 0;
static IRCClient *ircc = 0;


static void *reader(void *)
{
	char buf[256];

	while((!done) && fgets(buf, sizeof(buf), stdin))
	{
		Str::trim(buf);

		if(Str::startsWith(buf, "/"))
		{

			if(Str::startsWith(buf, "/join "))
			{
				ircc->join(buf + 6);
			}
			else
			if(Str::startsWith(buf, "/exit"))
			{
				break;
			}
			else
			{
				ircc->write(buf + 1);
			}
		}
		else
		{
			ircc->chat(buf);
		}

	}

	done = 1;

	return 0;
}


void p_chat(const char *from, const char *msg)
{
	printf("%s: %s\n", from, msg);
}


void p_info(const char *msg)
{
	printf("========= %s ======= \n", msg);
}

void p_here(const char *who)
{
	printf("========= Here:  %s  ======= \n", who);
}

void p_join(const char *who)
{
	printf("========= Join:  %s ======= \n", who);
}

void p_part(const char *who)
{
	printf("========= Part:  %s ======= \n", who);
}


int main(int argc, char **argv)
{
	
	if(argc != 4)
	{
		printf("Usage: %s host port nick\n", argv[0]);
		exit(1);
	}

	const char *host = argv[1];
	int port = atoi(argv[2]);
	const char *nick = argv[3];

	ircc = new IRCClient();
	
	if(!ircc->connect(host, port, nick))
	{
		printf("Can't connect to %s:%d\n", host, port);
		exit(1);
	}

	printf("Connected to %s:%d as %s\n", host, port, nick);

	Thread rt(reader);
	rt.start(0);

	while(!done)
	{
		while(ircc->read(p_info, p_chat, p_here, p_join, p_part))
			;
		System::ssleep(1);
	}

	Thread::join(&rt);

	return 0;
}




#endif



#endif // com_sleepless_net_ircclient_h
