#ifndef com_sleepless_mtserver_cpp
#define com_sleepless_mtserver_cpp

// Copyright 2008-2500 Sleepless Software Inc.  All Rights Reserved

/*
	This is code a multi-threaded server class.
	It creates a separate thread and object for each new connection.
*/


#include "log.cpp"
#include "thread.cpp"
#include "system.cpp"
#include "queue.cpp"
#include "serversocket.cpp"



/* extend this class to create your thread object */

struct MTSThread : Thread
{
	unsigned long id;
	Socket *sock;
	bool done;

	MTSThread()
	{
TRACE("MTSThread()\n");
		id = 0;
		sock = 0;
		done = false;
	}

	void start(unsigned long i, Socket *s)
	{
TRACE("MTSThread::start(i=%d, sock=%x)\n", i, s);
		id = i;
		sock = s;
		done = false;
		Thread::start();
	}

	void die()
	{
TRACE("MTSThread[%d]::die()\n", id);
		if(!done)
		{
TRACE("MTSThread[%d]::die() deleting sock %x\n", id, sock);
			done = true;
			delete sock;
			sock = 0;
		}
TRACE("MTSThread[%d]::die() dead\n", id);
	}

	void run()
	{
TRACE("MTSThread[%d]::run() enter\n", id);
		loop();
		die();
TRACE("MTSThread[%d]::run() exit\n", id);
	}

	/* override this method to provide you own functionality */
	virtual void loop()
	{
		// This is just an example, but thread must exit if done == true
		while(!done)
		{
			System::msleep(1000);
		}
	}
};


struct MTServer : Thread
{

	int port;
	int backlog;
	bool done;
	unsigned long thid;
	List list;


	void start(int p, int bl)
	{
TRACE("MTServer::start(%d, %d)\n", p, bl);
		done = false;
		thid = 0;
		list.remAll();
		port = p;
		backlog = bl;
		Thread::start();
	}


	void die()
	{
TRACE("MTServer::die() \n");
		if(!done)
		{
			done = true;
TRACE("MTServer::die() waking myself\n");
			delete new Socket("localhost", port);	// wake myself up
TRACE("MTServer::die() joining myself\n");
			join();
		}
TRACE("MTServer::die() dead\n");
	}


	void run()
	{
TRACE("MTServer::run() enter\n");
		done = false;
		ServerSocket ss(port, backlog);
		if(ss.isListening())
		{
TRACE("MTServer::run() listening on port %d\n", port);
			while(!done)
			{
				Socket *sock = ss.accept();
				if(!sock)
				{
TRACE("MTServer::run() accept returns 0 - breaking\n");
					break;
				}

				thid++;
TRACE("MTServer::run() new connection %d\n", thid);

				MTSThread *m = newThread();
TRACE("MTServer::run() adding new thread %x to list\n", m);
				list.append(m);
TRACE("MTServer::run() added - list len is now %d \n", list.length());
				m->start(thid, sock);
			}

TRACE("MTServer::run() out of loop - killing threads ...\n");
			while(true)
			{
				MTSThread *m = (MTSThread *)list.rem(0);
				if(!m)
				{
TRACE("MTServer::run()  no more threads - breaking\n");
					break;
				}
TRACE("MTServer::run()  killing thread %d ...\n", m->id);
				m->die();
TRACE("MTServer::run()  joining thread %d ...\n", m->id);
				m->join();
TRACE("MTServer::run()  joined thread %d ...\n", m->id);
			}
		}
TRACE("MTServer::run() exit\n");
	}


	
	/* This should return a newly created subclass of MTSThread or
		0 if one couldn't be created. */
	virtual MTSThread *newThread() = 0;

};




#ifdef MTSERVER_TEST


struct FooThread : MTSThread
{

	void loop()
	{
TRACE("FooThread[%d]::loop() enter\n", id);
		while(!done)
		{
			char s[256] = "";
			int r = sock->readString(s, sizeof(s));
TRACE("FooThread[%d]::loop() received: '%s' (%d bytes)\n", id, s, r);
			if(r == 0)
			{
TRACE("FooThread[%d]::loop() read error\n", id);
				break;
			}

			strcat(s, " (-:");
			int w = sock->write(s, strlen(s) + 1);
			if(w == -1)
			{
TRACE("FooThread[%d]::loop() write error\n", id);
				break;
			}
TRACE("FooThread[%d]::loop() wrote: '%s' (%d bytes)\n", id, s, w);
		}
TRACE("FooThread[%d]::loop() exit\n", id);
	}

};


struct MyServer : MTServer
{

	MTSThread *newThread()
	{
TRACE("MyServer::newThread()\n");
		return new FooThread();
	}

};


int main(int argc, char **argv)
{
	TRACEON();

	if(argc == 2)
	{
		int port = atoi(argv[1]);
		TRACE("Acting as server: port=%d\n", port);

		MyServer server;
		server.start(port, 5);
		System::msleep(30 * 1000);
		server.die();
	}
	else
	if(argc == 4)
	{
		const char *host = argv[1];
		int port = atoi(argv[2]);
		TRACE("Acting as client: host=%s port=%d\n", host, port);

		Socket sock(host, port);
		if(sock.isConnected())
		{
			TRACE("connected\n");
			char s[256];
			while(true)
			{
				int w = sock.write(argv[3], strlen(argv[3]) + 1);
				if(w != -1)
				{
					TRACE("wrote %d bytes\n", w);
					int r = sock.readString(s, sizeof(s));
					if(r != -1)
					{
						TRACE("read %d bytes\n", r);
						s[r] = 0;
						TRACE("response: '%s'\n", s);
					}
					else
					{
						TRACE("Read error\n");
						break;
					}
				}
				else
				{
					TRACE("Write error\n");
					break;
				}
				System::msleep(1000);
			}
		}
		else
			TRACE("Couldn't connect\n");

	}

	TRACE("Done.\n");

	return 0;
}


#endif





#endif // com_sleepless_mtserver_cpp
