#ifndef com_sleepless_net_sslserversocket_h
#define com_sleepless_net_sslserversocket_h

// Copyright 2002
// Sleepless Software Inc.
// All Rights Reserved



#include "ServerSocket.cpp"
#include "SSLSocket.cpp"


static DH *sslGetDH()
{
	static unsigned char dh1024_p[]={
		0xC2,0x22,0x6C,0x5B,0xA5,0x19,0x7C,0x9C,0x0C,0x7F,0x9E,0xBF,
		0x6D,0x94,0xEC,0x08,0xF4,0xD6,0x0B,0x80,0x2A,0x01,0xD4,0x33,
		0xF1,0xC1,0x0D,0x2B,0x10,0xB8,0xDA,0xB8,0x6D,0x43,0x47,0x5F,
		0xDB,0x77,0x53,0xFD,0x99,0x79,0xBD,0xFB,0x9C,0x22,0x2E,0xCB,
		0xED,0x81,0xD3,0x9D,0xFD,0x8E,0xC0,0xA4,0x4D,0x41,0x26,0x0B,
		0xB8,0xD5,0xC6,0xA5,0x3A,0x1A,0x7E,0xBC,0x74,0x16,0x3E,0x2E,
		0x3D,0x99,0xDE,0x08,0xD3,0x92,0xA1,0x86,0x3C,0xEB,0x0B,0x43,
		0xC5,0x99,0x56,0x1E,0xB4,0x86,0x7E,0xFA,0x4E,0x9B,0x7F,0x85,
		0xC3,0x10,0x69,0xD9,0x8E,0x45,0xBF,0x69,0x63,0xF5,0x87,0xAC,
		0x8F,0x71,0x06,0xE5,0x50,0xBC,0x7F,0xB8,0xBE,0xDD,0xD1,0x3B,
		0x7D,0xD6,0xA1,0xD3,0xAD,0x1A,0x85,0x83,
		};
	static unsigned char dh1024_g[]={
		0x02,
		};
	DH *dh;
	if ((dh=DH_new()) == NULL) return(NULL);
	dh->p=BN_bin2bn(dh1024_p,sizeof(dh1024_p),NULL);
	dh->g=BN_bin2bn(dh1024_g,sizeof(dh1024_g),NULL);
	if ((dh->p == NULL) || (dh->g == NULL))
		{ DH_free(dh); return(NULL); }
	return(dh);
}


struct SSLServerSocket : ServerSocket
{

	bool ssInit(const char *certfile, const char *keyfile)
	{
		// load server certificate
		if(!(SSL_CTX_use_certificate_file(ctx, certfile, SSL_FILETYPE_PEM)))
		{
			sslTrouble("Couldn't read certificate file");
			return false;
		}

		// load server's private key 
		if(keyfile)
		{
			if(!(SSL_CTX_use_PrivateKey_file(ctx, keyfile, SSL_FILETYPE_PEM)))
			{
				sslTrouble("Couldn't read key file");
				return false;
			}
		}

		// Do the mysterious (to me) Diffie-Hellman magic
		DH *dh = sslGetDH();
		if(SSL_CTX_set_tmp_dh(ctx, dh) < 0)
		{
			sslTrouble("Couldn't set DH parameters");
			return false;
		}

		// Generate rsa key pair
		RSA *rsa = RSA_generate_key(512, RSA_F4, NULL, NULL);
		if(!SSL_CTX_set_tmp_rsa(ctx, rsa))
		{
			sslTrouble("Couldn't set RSA key");
			return false;
		}
		RSA_free(rsa);

		// Set up server context
		// XXX should include PID or something a little more unique.
		const unsigned char *srv_context = (const unsigned char *)"1";
		SSL_CTX_set_session_id_context(ctx, srv_context, strlen((const char *)srv_context));

		return true;
	}


	SSLServerSocket(int port, int backlog, const char *randfile, const char *certfile, const char *keyfile) : ServerSocket(port, backlog)
	{
		if(isListening())
		{
			sslInit(randfile, NULL);
			ssInit(certfile, keyfile);
		}
	}


	~SSLServerSocket()
	{
		serverSocketClose(ss);
		ss = INVALID_SOCKET;
	}


	SSLSocket *accept()
	{
		struct sockaddr_in sa;
		memset(&(sa), 0, sizeof(sa)); 
		socklen_t salen = sizeof(sa);
		SOCKET s = serverSocketAccept(ss, &sa, &salen);
		if(s == INVALID_SOCKET)
			return NULL;
		unsigned long addr = ntohl(sa.sin_addr.s_addr);
		return new SSLSocket(s, addr, NULL, NULL);
	}


	int pending()
	{
		return serverSocketPending(ss);
	}

};



#ifdef SSLSERVERSOCKET_TEST

#include <stdio.h>
#include <stdlib.h>

int main(int argc, char **argv)
{
	if(argc < 5)
	{
		printf("Usage: %s port randfile certfile keyfile\n", argv[0]);
		return 1;
	}

	int port = atoi(argv[1]);
	char *randfile = argv[2];
	char *certfile = argv[3];
	char *keyfile = argv[4];
	if(keyfile[0] == 0)
		keyfile = 0;

	SSLServerSocket ss(port, 1, randfile, certfile, keyfile);
	if(!ss.isListening())
	{
		printf("#### Can't create SSLServerSocket on port %d\n", port);
		return 1;
	}

	printf("Listening on port %d\n", port);

	SSLSocket *sock = ss.accept();
	if(!sock)
	{
		printf("#### Bad accept()\n");
		return 1;
	}

	printf("Got connection ...\n");

	char line[256];

	int r = sock->readLine(line, sizeof(line));

	if(r == -1)
	{
		printf("#### readLine() returned -1\n");
		exit(1);
	}

	if(r == 0)
	{
		printf("Got EOF\n");
		exit(1);
	}

	printf("read: %s", line);

	r = sock->writeStr("Hello. I'm the server. You said: ");
	r = sock->writeStr(line);

	if(r == -1)
	{
		printf("#### sock->writeStr() returned -1\n");
		exit(1);
	}

	if(r == 0)
	{
		printf("#### sock->writeStr() returned 0\n");
		exit(1);
	}

	delete sock;

	printf("closed\n");

	return 0;
}

#endif


#endif	// com_sleepless_net_sslserversocket_h

