#ifndef com_sleepless_net_http_httpgetsimplessl_cpp
#define com_sleepless_net_http_httpgetsimplessl_cpp

// Copyright 2002
// Sleepless Software Inc.
// All Rights Reserved




#include <stdlib.h>

#include "../SSLSocket.cpp"

#ifdef WIN32
#include "win32_httpgetrequest.h"
#endif


int httpGetSimpleSSL(const char *h, int p, const char *path, char *buf, int len, const char *cafile)
{
#ifdef WIN32
		
	const char *cafile_foo = cafile;		// just to avoid compiler warning

	
	//This is the lib function, that takes the host, port, path, SSL? and if it 
	//should be reloaded and finally if it should be cached.
	HTTPGetRequest theSSLRequest( h, p, path, true, false, false);
	if ( theSSLRequest.failed )
		return 0;

	unsigned long numBytesRead = 0;		//The total number of bytes read from the connection.
	int	numLoopsNoRead = 0;				//Our loop safety, limits the number of times
										//we'll spin without reading anything on the port.
	int numThisLoop = 0;				//How many bytes read in the current iteration.

	//Loop safety.	
	#define kMaxLoopsWithoutRead 300

	//While there is stuff to read...
	while ( theSSLRequest.leftToRead > 0 )
	{
		//Read from the socket.
		numThisLoop = theSSLRequest.read(buf, len);

		//Check to see if we are done.
		if (theSSLRequest.failed)
		{
			numBytesRead = 0;
			break;
		}

		//See if we are done.
		if ( numThisLoop == 0 )
		{
			numBytesRead = 0;
			break;
		}

		//Up the count read.
		numBytesRead += numThisLoop;

		//Clear the count for this iteration.
		numThisLoop = 0;
	}

	return numBytesRead;

#else

//TRACE("%s-%d\n",__FILE__, __LINE__);

	SSLSocket s(h, p, 0, cafile);
	if(!s.isConnected())
	{
//TRACE("%s-%d\n",__FILE__, __LINE__);
		return -1;
	}

	s.writeStr("GET ");
	s.writeStr(path);
	s.writeStr(" HTTP/1.0\n");
	s.writeStr("\n");

	char line[200];
	int r = 0;
	long contentLength = 0;
	while(true)
	{
		if(s.readLine(line, sizeof(line)) < 1)
			break;

		if(strcmp(line, "\n") == 0)
		{
			int l = len;
			if(contentLength > 0)
			{
				if(contentLength < len)
				{
					l = contentLength;
				}
			}
			r = s.read(buf, l);
			break;
		}

		if(strncasecmp(line, "Content-Length: ", 16) == 0)
		{
			contentLength = atol(line + 16);
		}
	}

	return r;

#endif
	
}


int httpGetSimpleSSL(const char *h, int p, const char *path, char *buf, int len)
{
	return httpGetSimpleSSL(h, p, path, buf, len, "ca.crt");
}



#ifdef TEST

#include <stdio.h>


int main(int argc, char **argv)
{
	if(argc < 5)
	{
		printf("Usage: %s host port path cafile\n", argv[0]);
		return 1;
	}

	char *host = argv[1];
	int port = atol(argv[2]);
	char *path = argv[3];
	char *cafile = argv[4];

	char buf[20000];
	int r = httpGetSimpleSSL(host, port, path, buf, sizeof(buf), cafile);
	if(r == -1)
	{
		fprintf(stderr, "Error reading from \"https://%s:%d/%s\" (cafile=%s)\n", host, port, path, cafile);
		return 1;
	}
	fwrite(buf, 1, r, stdout);
	return 0;
}

#endif


#endif	// com_sleepless_net_http_httpgetsimplessl_cpp
