#ifndef com_sleepless_prefs_cpp
#define com_sleepless_prefs_cpp

// Copyright 2008  Sleepless Software Inc.  All Rights Reserved

/* Implements automatic, persistant storage of tagged values.
	Uses a filesystem path for storage which is required for creation.
	Includes support for default values to be returned if tags haven't
	had a value set yet.
	*/

#include "str.cpp"
#include "taglist.cpp"


struct Prefs
{
	char path[300];
	struct TagList tl;
	bool dirty;

	/* 'p' is the path to the file where you'd like the prefs stored */
	Prefs(const char *p)
	{
//TRACE("Prefs(%s)\n", p);
		dirty = false;
		STR_CPY(path, p);
		tl.readFile(path);
	}

	~Prefs()
	{
		if(dirty)
			flush();
	}


	bool isSet(const char *tag)
	{
		return (tl.get(tag)) ? true : false;
	}


	/* Returns the value with the given tag as a string,
		or 'def' if the value has never been set */
	const char *getStr(const char *tag, const char *def = 0)
	{
		const char *val = tl.get(tag);
		if(!val && def)
			tl.set(tag, def);
		return tl.get(tag);
	}

	/* Returns the value with the given tag as an integer,
		or 'def' if the value has never been set */
	int getInt(const char *tag, int def = 0)
	{
		const char *val = tl.get(tag);
		if(!val)
			tl.set(tag, def);
		return tl.getInt(tag);
	}

	/* Sets a string value */
	void setStr(const char *tag, const char *val, bool f = false)
	{
		tl.set(tag, val);			// set it
		if(f)
			flush();
		dirty = true;
	}

	/* ditto */
	void set(const char *tag, const char *val, bool f = false)
	{
		setStr(tag, val, f);
	}


	/* Sets an integer value */
	void setInt(const char *tag, int val, bool f = false)
	{
		tl.set(tag, val);			// set it
		if(f)
			flush();
		dirty = true;
	}

	/* ditto */
	void set(const char *tag, int val, bool f = false)
	{
		setInt(tag, val, f);
	}

	void flush()
	{
		tl.writeFile(path);			// flush to persistant storage
		dirty = false;
	}
};


#endif	// com_sleepless_prefs_cpp
