#ifndef com_sleepless_misc_queue_cpp
#define com_sleepless_misc_queue_cpp

// Copyright 1998-2304  Sleepless Software Inc.  All Rights Reserved

/* This is a first in, first out queue that holds pointers to whatever.
	It's also mutex protected, so it is thread safe, unlike most of the
	other stuff around here. 
	*/


#include "mutex.cpp"
#include "list.cpp"


struct Queue
{
	List list;
	Mutex mutex;


	// Return the number of items currently in the queue.
	int length()
	{
		int n;
		mutex.lock();
		n = list.length();
		mutex.unlock();
		return n;
	}

	// Add a pointer to the end of the queue.
	void queue(void *p)
	{
		mutex.lock();
		list.append(p);
		mutex.unlock();
	}

	// Remove and return the pointer from the head of the queue.
	// If the queue is empty, NULL is returned.
	void *next()
	{
		void *p = NULL;
		mutex.lock();
		p = list.rem(0);
		mutex.unlock();
		return p;
	}

	// Return the pointer at index 'i' without removing it from the queue.
	void *at(int i)
	{
		void *p = NULL;
		mutex.lock();
		p = list.get(i);
		mutex.unlock();
		return p;
	}

	void clear()
	{
		mutex.lock();
		list.remAll();
		mutex.unlock();
	}

};






#ifdef TESTING

#include <stdio.h>


void dump(List &list)
{
	int l = list.length();
	printf("  length() = %d\n", l);
	for(int i = 0; i < l; i++)
	{
		char *s = (char *)list.get(i);
		printf("  %d: %s\n", i, s);
	}
}

int main(int argc, char **argv)
{
	Queue q;

	for(int i = 1; i < argc; i++)
	{
		printf("queueing %s ...\n", argv[i]);
		q.queue(argv[i]);
	}

	printf("raw list:\n");
	dump(q.list);

	char *foo = "foo";
	char *bar = "bar";
	char *baz = "baz";

	printf("queueing foo, bar, and baz:\n");
	q.queue(foo);
	q.queue(bar);
	q.queue(baz);
	dump(q.list);

	char *x = (char *)q.at(1);
	printf("item at index 1 is: %s\n", x);

	printf("removing next: ...");
	x = (char *)q.next();
	printf("%s\n", x);
	dump(q.list);

	printf("removing all:\n");
	q.clear();
	dump(q.list);

}

#endif


#endif // com_sleepless_misc_queue_cpp
