#ifndef com_sleepless_scramble_cpp
#define com_sleepless_scramble_cpp

// Copyright 2006 Sleepless Software Inc.  All Rights Reserved

/*

	The routines in this file:

		static void scramble(unsigned char *buf, int len)
		static void unscramble(unsigned char *buf, int len)

	These routines are use to effectively hide/disguise the bytes
	in a buffer.  They combine rot13 (see rot13.cpp) and
	cloak (see cloak.cpp) to hopefully provide a rather obscure
	scrambling of the data, such that it would be very hard for
	someone to sort out without some serious effort.

*/


#include <string.h>

#include "rot13.cpp"
#include "cloak.cpp"


struct Scramble
{
	static void scramble(unsigned char *buf, int len)
	{
		rot13(buf, len);
		cloak(buf, len);
	}

	static void unscramble(unsigned char *buf, int len)
	{
		decloak(buf, len);
		rot13(buf, len);
	}
};


#ifdef TEST_SCRAMBLE

#include <stdio.h>

static void hexdump(const unsigned char *b, int l)
{
	int len = l;
	const unsigned char *buf = b;

	while (--len >= 0) {
		printf(" %02X", *buf++);
	}

	printf(" ");

	len = l;
	buf = b;

	while (--len >= 0) {
		if(*buf < 32 || *buf > 126)
			printf(".");
		else
			printf("%c", *buf++);
	}

	printf("\n");
}

int main(int argc, char **argv)
{
	unsigned char *buf = (unsigned char *)argv[1];
	int l = strlen((char *)buf);
	printf("original   :"); hexdump(buf, l);
	scramble(buf, l);
	printf("scrambled  :"); hexdump(buf, l);
	unscramble(buf, l);
	printf("unscrambled:"); hexdump(buf, l);
}
#endif


#endif		// com_sleepless_scramble_cpp
