#ifndef	com_sleepless_sleepysdl_cpp
#define	com_sleepless_sleepysdl_cpp


// I only want to use SDL under linux, even though SDL supports win32
#if defined(linux) || (defined(__APPLE__) && defined(__GNUC__))


#include "assert.cpp"
#include "log.cpp"

#include "/usr/include/SDL/SDL.h"


struct SleepySDL
{
	SDL_Surface *bbSurf;			// The offscreen screen buffer
	SDL_PixelFormat *bbPixFmt;	// Pixel format
	int dispw, disph;
	const char *titlew, *titlei;

	SleepySDL()
	{
		dispw = 0;
		disph = 0;
		titlew = 0;
		titlei = 0;

		int r = SDL_Init(SDL_INIT_TIMER);
		DASSERT(r != -1);

		//version = SDL_Linked_Version();
		//m(("SDL runtime: %d.%d.%d", version->major, version->minor, version->patch));
	}


	void init_video(int w, int h, const char *tw, const char *ti)
	{
		dispw = w;
		disph = h;
		titlew = strdup(tw);
		titlei = strdup(ti);

		int r = SDL_InitSubSystem(SDL_INIT_VIDEO);
		DASSERT(r != -1);

		const SDL_VideoInfo *vi = SDL_GetVideoInfo();
		TRACE("Can create hardware surfaces? %s\n",(vi->hw_available)?"Y":"N");
		TRACE("Window manager available? %s\n",(vi->wm_available)?"Y":"N");
		TRACE("Accelerated hw to hw blits? %s\n",(vi->blit_hw)?"Y":"N");
		TRACE("Accelerated hw to hw colorkey blits? %s\n",(vi->blit_hw_CC)?"Y":"N");
		TRACE("Accelerated hw to hw alpha blits? %s\n",(vi->blit_hw_A)?"Y":"N");
		TRACE("Accelerated sw to hw blits? %s\n",(vi->blit_sw)?"Y":"N");
		TRACE("Accelerated sw to hw colorkey blits? %s\n",(vi->blit_sw_CC)?"Y":"N");
		TRACE("Accelerated sw to hw alpha blits? %s\n",(vi->blit_sw_A)?"Y":"N");
		TRACE("Accelerated color fills? %s\n",(vi->blit_fill)?"Y":"N");
		TRACE("Video memory %dKB\n",vi->video_mem);
		TRACE("Bits per pixel %d\n", vi->vfmt->BitsPerPixel);
		TRACE("Bytes per pixel %d\n", vi->vfmt->BytesPerPixel);
		TRACE("Color key (transparent pixel) %d\n", vi->vfmt->colorkey);
		TRACE("Alpha %d\n", vi->vfmt->alpha);
		char vd[80];
		SDL_VideoDriverName(vd, sizeof(vd));
		TRACE("Driver: %s\n", vd);

		// Store pointer to pixel format of display
		bbPixFmt = vi->vfmt;


		// Get available screen modes
		SDL_Rect **modes;
		int i;
		modes = SDL_ListModes(NULL, SDL_FULLSCREEN);
		if(modes == (SDL_Rect **)0)
		{
			TRACE("No fullscreen modes available!\n");
		}
		if(modes == (SDL_Rect **)-1)
		{
			TRACE("All fullscreen resolutions available.\n");
		}
		else
		{
			TRACE("Available fullscreen modes:\n");
			for(i=0;modes[i];++i)
				TRACE("  %d x %d\n", modes[i]->w, modes[i]->h);
		}


		Uint32 bpp;
		bpp = SDL_VideoModeOK(dispw, disph, 16, SDL_HWSURFACE);
		DASSERT(bpp);

		TRACE("SDL Recommends 640 x 480 @ %d bpp.\n", bpp);
		bbSurf = SDL_SetVideoMode(dispw, disph, bpp, SDL_HWSURFACE | SDL_DOUBLEBUF);
		TRACE("Achieved:\n");
		TRACE("  res: %d x %d\n", bbSurf->w, bbSurf->h);
		TRACE("  pitch: %d\n", bbSurf->pitch);
		TRACE("  mem: %s\n", bbSurf->flags & SDL_SWSURFACE ? "sys" : "vid");
		TRACE("  asyncblit: %s\n", bbSurf->flags & SDL_ASYNCBLIT ? "Y" : "N");
		TRACE("  dblbuf: %s\n", bbSurf->flags & SDL_DOUBLEBUF ? "Y" : "N");
		TRACE("  fullscr: %s\n", bbSurf->flags & SDL_FULLSCREEN ? "Y" : "N");
		TRACE("  sizable: %s\n", bbSurf->flags & SDL_RESIZABLE ? "Y" : "N");
		TRACE("  hwaccel: %s\n", bbSurf->flags & SDL_HWACCEL ? "Y" : "N");
		TRACE("  clrkey: %s\n", bbSurf->flags & SDL_SRCCOLORKEY ? "Y" : "N");
		TRACE("  rleaccel: %s\n", bbSurf->flags & SDL_RLEACCEL ? "Y" : "N");
		TRACE("  alphablend: %s\n", bbSurf->flags & SDL_SRCALPHA ? "Y" : "N");
		TRACE("  prealloced: %s\n", bbSurf->flags & SDL_PREALLOC ? "Y" : "N");
		TRACE("  format:\n");
		TRACE("    bitsperpixel: %d\n", bbSurf->format->BitsPerPixel);
		TRACE("    bytesperpixel: %d\n", bbSurf->format->BytesPerPixel);
		TRACE("    rmask: %x\n", bbSurf->format->Rmask);
		TRACE("    gmask: %x\n", bbSurf->format->Gmask);
		TRACE("    bmask: %x\n", bbSurf->format->Bmask);
		TRACE("    amask: %x\n", bbSurf->format->Amask);
		TRACE("    rshift: %x\n", bbSurf->format->Rshift);
		TRACE("    gshift: %x\n", bbSurf->format->Gshift);
		TRACE("    bshift: %x\n", bbSurf->format->Bshift);
		TRACE("    ashift: %x\n", bbSurf->format->Ashift);
		TRACE("    rloss: %x\n", bbSurf->format->Rloss);
		TRACE("    gloss: %x\n", bbSurf->format->Gloss);
		TRACE("    bloss: %x\n", bbSurf->format->Bloss);
		TRACE("    aloss: %x\n", bbSurf->format->Aloss);
		TRACE("    clrkey: %d\n", bbSurf->format->colorkey);
		TRACE("    alpha: %d\n", bbSurf->format->alpha);


//		SDL_EnableUNICODE(true);


		SDL_WM_SetCaption(titlew, titlei);
	}


	// Prepare for rendering into back buffer
	void draw_prep()
	{
		bbSurf = SDL_GetVideoSurface();
	}


	// Copy contents of the back buffer to the real display
	void draw_flush()
	{
		SDL_Flip(bbSurf);
	}



#if 0
	void init_audio()
	{
		int r = SDL_InitSubSystem(SDL_INIT_AUDIO);
		DASSERT(r != -1);
		SDL_AudioSpec desired;
		desired.freq = 22050;				// 22050Hz - FM Radio quality 
		desired.format = AUDIO_S16LSB;		// 16-bit signed audio 
		desired.channels = 1;
		desired.silence = 0;
		desired.samples = 1024;
		desired.size = 0;
		desired.callback = fillAudio;
		desired.userdata = NULL;
		SDL_AudioSpec audioSpec;
		r = SDL_OpenAudio(&desired, &audioSpec);
		DASSERT(r == 0);
		SDL_PauseAudio(0);		// start mixing and playing audio
	}

	void fillAudio(void *udata, unsigned char *stream, int len)
	{
		// Mix in all sound channels.
		for(int i = 0; i < NUM_CHANNELS; i++)
		{
			Sound *snd = soundChannels[i].sound;
			if(snd != NULL)
			{
				unsigned int pos = soundChannels[i].bufpos;	// cur index into sample
				int remain = snd->length - pos;		// # of bytes left to play
				if(remain > 0)
				{
					// Mix in either full buffer size, or amount remaining for
					// this sound, whichever is less.
					int amt = (remain < len) ? remain : len;

					mdMixAudio(stream, &(snd->buffer[pos]), amt, soundChannels[i].volume);

					soundChannels[i].bufpos += amt;	// bump up index into snd buf
				}
				if(pos >= snd->length)
				{
					// The sound's been fully played, so clear out it's channel.
					soundChannels[i].bufpos = 0;
					soundChannels[i].volume = 0.0;
					soundChannels[i].sound = NULL;	// MUST DO THIS LAST
				}
			}
		}
	}
#endif

	
};


#ifdef TEST_SLEEPYSDL

int main(int argc, char **argv)
{
	TRACEON();

	SleepySDL ssdl;

	ssdl.init_video(300, 200, "Window Title", "Icon Title");

	getchar();
	
	return 0;
}


#endif


#endif


#endif		//	com_sleepless_sleepysdl_cpp
