#ifndef com_sleepless_string_h
#define com_sleepless_string_h

// Copyright 3000 Sleepless Software Inc.  All Rights Reserved

#include <string.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>

//#include "assert.cpp"
#include "log.cpp"

#ifdef WIN32
#define strcasecmp	_stricmp
#endif

#ifdef WIN32
#define strcasecmp	_stricmp
#endif

static int nseq = 0;

typedef struct String
{
	char *chars;
	int len; 
	int seq;


	void set(const char *s, int i, int l)
	{
		int sl = (int)strlen(s);

		if(l < 0)
			l = 0;
		if(i < 0)
			i = sl + i;
		if(i < 0)
			i = 0;
		if(i > sl)
			i = sl;
		if(i + l > sl)
			l = sl - i;
		if(chars)
			delete [] chars;
		chars = new char[l+1];
		strncpy(chars, s+i, l);
		chars[l] = 0;
		len = l;
		// TRACE("   s-%d \"%s\"\n", seq, chars);
	}

	void set(const char *s)
	{
		set(s, 0, (int)strlen(s));
	}

	void init(const char *s)
	{
		chars = 0;
		len = 0;
		seq = ++nseq;
#ifdef TEST_STRING
		if(seq >= 10) exit(1);
#endif
		set(s);
	}

	String() { init(""); }

	~String() { set("~String()"); delete [] chars; }

	String & operator=(const String &s2) { if (&s2 != this) set(s2.chars); return *this; }
	String & operator=(const char *cs) { set(cs); return *this; }

	String(String &s2) { init(""); *this = s2; }
	String(
			const char *cs,
			const char *cs2 = 0,
			const char *cs3 = 0,
			const char *cs4 = 0,
			const char *cs5 = 0
			)
	{
		init("");
		*this = cs;
		if(cs2)
			this->cat(cs2);
		if(cs3)
			this->cat(cs3);
		if(cs4)
			this->cat(cs4);
		if(cs5)
			this->cat(cs5);
	}
	String(int n) { init(""); this->cat(n); }
//	String(int n) { init(""); *this = n; }
//	String(double n) { init(""); *this = n; }

	operator const char *() const { return chars; }
//	operator int () const { return atoi(chars); }
//	operator double () const { return atof(chars); }

	bool operator==(const char *s) const { return strcasecmp(chars, s) == 0; }
	bool operator!=(const char *s) const { return strcasecmp(chars, s) != 0; }

	char operator[](int i) { if(i < 0) i = len + i; return chars[i]; }


	String & cat(const char *s)
	{
		char *nc = new char[this->len + strlen(s) + 1];
		strcpy(nc, this->chars);
		strcat(nc, s);
		set(nc);
		delete nc;
		return *this; 
	}

	String & cat(char c)
	{
		char tmp[2];
		tmp[0] = c;
		tmp[1] = 0;
		return cat(tmp);
	}

	String & cat(int n)
	{
		char num[30];
		sprintf(num, "%d", n);
		return cat(num);
	}


	String & clip(int i, int l)
	{
		String tmp(chars);
		this->set(tmp.chars, i, l);
		return *this;
	}

	String & clip(int i)
	{
		return clip(i, len);
	}

	String & trim()
	{
		char *cp = &chars[len - 1];
		while(isspace(*cp))
			*cp-- = 0;
		cp = chars;
		while(isspace(*cp))
			cp++;
		set(cp);
		return *this;
	}


	bool startsWith(const char *pre)
	{
		String tmp(chars);
		tmp.clip((int)strlen(pre));
		return tmp == pre;
	}

	bool endsWith(const char *suf)
	{
		String tmp(chars);
		tmp.clip((int)strlen(suf) * -1);
		return tmp == suf;
	}

	bool contains(const char *pat)
	{
		String s1(chars); s1.lower();
		String s2(pat); s2.lower();
		return strstr(s1.chars, s2.chars) != 0;
	}

	/* changes string to all lower case */
	void lower()
	{
		char *t = chars;
		while(*t)
		{
			*t = tolower(*t);
			t++;
		}
	}

	/* replace all occurances of 'o' with 'n' */
	String & chrepl(int o, int n)
	{
		char *t = chars;
		while(*t)
		{
			if(*t == o)
				*t = n;
			t++;
		}
		return *this;
	}

	/* return index of first occurance of 'c' */
	int indx(int c)
	{
		int i = 0;
		while(chars[i])
		{
			if(chars[i] == c)
				return i;
			i++;
		}
		return -1;
	}

	/* return index of last occurance of 'c' */
	int rindx(int c)
	{
		int i = len - 1;
		while(i >= 0)
		{
			if(chars[i] == c)
				return i;
			i--;
		}
		return -1;
	}

	/* cuts off tail of string (leaving head) at the first occurance of 'c' (including 'c') */
	void head(int c)
	{
		int i = indx(c);
		if(i >= 0)
			this->clip(0, i);
	}

	/* cuts off head of string (leaving tail) at the first occurance of 'c' (including 'c') */
	void tail(int c)
	{
		int i = indx(c);
		if(i >= 0)
			this->clip(i+1);
	}

	/* cuts off tail of string (leaving head) at the last occurance of 'c' (including 'c') */
	void rhead(int c)
	{
		int i = rindx(c);
		if(i >= 0)
			this->clip(0, i);
	}

	/* cuts off head of string (leaving tail) at the last occurance of 'c' (including 'c') */
	void rtail(int c)
	{
		int i = rindx(c);
		if(i >= 0)
			this->clip(i+1);
	}


} S;


#ifdef TEST_STRING

#include <math.h>

typedef String S;

int main(int argc, char **argv)
{
	S s;
	s = "  \tboobs are fun! \n";
	s.trim();
	s.clip(1,90);
	s.clip(1,7);
	s.clip(4);
	s.clip(-2, 1);
	s.cat("ufus the dog").cat(" smells").cat(" really bad");
	printf("c='%c'\n", s[2]);
	s.clip(0, 5);

	if(s != "rufus") printf("not rufus\n"); else printf("yes rufus!\n");
	if(s != "goofus") printf("not goofus\n"); else printf("yes goofus!\n");

	S g("RUFus");
	if(s == g) printf("hi roofy\n");

	s = 11;
	s = 22.0f;
	s = "some teXt = ";
	s.cat(S(33));

	s = 44;
	g = 55;

	s = int(s) + int(g);
	s = int(s) + 1;
	s = double(s) / 3;
	s = double(s) + int(g);

	g = floor(s) / 2;

	s = "hello %s #%d!\n";
	g = "world";
	printf(s, (const char *)g, 3);

	
	s = "  foo bar   ";
	s.trim().clip(2,3);

	return 0;
}


#endif

#endif // com_sleepless_string_h
