
#ifdef WIN32

#	include "mutex.cpp"

	struct Unilock
	{
		Mutex *mutex;
		bool lck;

		Unilock(const char *n)
		{
			lck = 0;
			mutex = new Mutex(n);
			if(mutex)
			{
				lck = mutex->tryLock(1) == 0;
				if(!lck)
				{
					delete mutex;
					mutex = 0;
				}
			}
		}

		virtual bool locked()
		{
			return lck;
		}

		~Unilock()
		{
			if(mutex)
				delete mutex;
		}

	};

#else

#	include <sys/un.h>

#	include "file.cpp"
#	include "socket.cpp"

	struct Unilock
	{
		int sock;
		char name[100];

		Unilock(const char *n)
		{
			sock = INVALID_SOCKET;
			STR_CPY(name, n);

			File f(name);
			if(f.exists())
			{
				if((time(0) - f.getMTime()) > (3600 * 24))
					unlink(name);
			}

			int len;
			sockaddr_un addr;
			sock = socket(AF_UNIX, SOCK_STREAM, 0);
			addr.sun_family = AF_UNIX;
			strcpy(addr.sun_path, name);
			len = sizeof(addr);
			if(bind(sock, (sockaddr*)&addr, len) != SOCKET_ERROR)
				return;
			DTLOG("bind fail %s\n", name);
			closesocket(sock);
			sock = INVALID_SOCKET;
		}

		virtual bool locked()
		{
			return sock != INVALID_SOCKET;
		}

		~Unilock()
		{
			if(sock != INVALID_SOCKET)
			{
				closesocket(sock);
				unlink(name);
			}
		}

	};

#endif

