#ifndef com_sleepless_net_http_urlencoder_cpp
#define com_sleepless_net_http_urlencoder_cpp

// Copyright 1998-2007
// Sleepless Software Inc.
// All Rights Reserved


#include <string.h>
#include <stdio.h>




struct URLEncode
{

	/*
	Arguments 's' and 'o' must NOT be null.
	's' is the string to encode.
	'o' is the destination for the encoded string.
	Note that the output can be as much as 3 times the original size
	so you need to pass in a buffer for 'o' that is at least 3 times
	the size of the input string.

	When encoding a String, the following rules apply:
	* The space character " " is converted into a plus sign "+".
	* The alphanumeric characters "a" through "z", "A" through "Z" and
	"0" through "9" remain the same.
	* The special characters ".", "-", "*", and "_" remain the same.
	* All other characters are unsafe and are first converted into one
	or more bytes using some encoding scheme. Then each byte is
	represented by the 3-character string "%xy", where xy is the
	two-digit hexadecimal representation of the byte. The recommended
	encoding scheme to use is UTF-8. However, for compatibility
	reasons, if an encoding is not specified, then the default
	encoding of the platform is used.
	*/
	static void encode(const char *s, char *o) 
	{
		while(*s)
		{
			if((*s >= 'a') && (*s <= 'z'))
			{
				*o++ = *s++;
			}
			else
			if((*s >= 'A') && (*s <= 'Z'))
			{
				*o++ = *s++;
			}
			else
			if((*s >= '0') && (*s <= '9'))
			{
				*o++ = *s++;
			}
			else
			if(*s == ' ')
			{
				*o++ = '+';
				s++;
			}
			else
			if((*s == '.') || (*s == '-') || (*s == '*') || (*s == '_'))
			{
				*o++ = *s++;
			}
			else
			{
				sprintf(o, "%%%2.2x", (int)(*s & 0xff));
				o += 3;
				s++;
			}
		}
		*o = 0;
	}


	/*
	Argument 's' must NOT be null.
	's' is the string to decode.
	'o' is the destination for the decoded string.
	Returns 0 if decoding went normally.
	Returns -1 if there was any bogosity detected in the encoding.
	NOTE: Because of the way this function is written, and because decoding
	effectively shrinks (or at least doesn't grow) the output, it is
	safe to call this function with the output 'o' being the same
	string as the input 's'.  Doing so results in the input string 
	being decoded "in place".
	*/
	static int decode(char *s, char *o) 
	{
		int n;
		while(*s)
		{
			if(*s == '+')
			{
				*o++ = ' ';
				s++;
			}
			else
			if(*s == '%')
			{
				if(strlen(s) < 3)
					return -1;
				if(sscanf(s, "%%%2x", &n) != 1)
					return -1;
				*o++ = (char)(n & 0xff);
				s += 3;
			}
			else
			{
				*o++ = *s++;
			}
		}
		*o = 0;
		return 0;
	}


	// returns true if the parameter was appended to the request.
	// false if 'req' isn't big enough to accept the parameter.
	// assumes the initial "?" is already present.
	static bool addParam(char *req, int max, const char *param, const char *tag)
	{
		char *eparam = (char *)new char[(strlen(param) * 3) + 1];
		if(!eparam)
			return false;
		encode(param, eparam);
		if(((int)strlen(req) + (int)strlen(tag) + 1 + (int)strlen(eparam) + 1 + 1) > max)
		{
			delete [] eparam;
			return false;
		}
		strcat(req, tag);
		strcat(req, "=");
		strcat(req, eparam);
		strcat(req, "&");
		delete [] eparam;
		return true;
	}

	static bool addParam(char *req, int max, int param, const char *tag)
	{
		char num[50];
		sprintf(num, "%d", param);
		return addParam(req, max, num, tag);
	}

	static bool addParam(char *req, int max, float param, const char *tag)
	{
		char num[50];
		sprintf(num, "%f", param);
		return addParam(req, max, num, tag);
	}

};


#ifdef TEST_URLE
#include <stdio.h>
int main(int argc, char **argv)
{


	for(int i = 1; i < argc; i++)
	{
		char *a = argv[i];
		char *out = new char[strlen(a)];

		printf("argument %d\n", i);
		printf("original: '%s'\n", a);
		URLEncode::encode(a, out);
		printf("encoded : '%s'\n", out);
		if(URLEncode::decode(out, out))
			printf("Decode error\n");
		else
			printf("decoded : '%s'\n", out);
		printf("\n");

		delete out;
	}

	return 0;
}
#endif


#endif
